import { system, world, Player } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

system.beforeEvents.watchdogTerminate.subscribe(eventData => {
    console.error("watchdog = " + eventData.terminateReason +
        " trying to cancel crash")
    eventData.cancel = true
});

world.beforeEvents.chatSend.subscribe(data => {
    data.cancel = true
    const rank = getRanks("rank:", data.sender).length <= 0 ? "§9Oyuncu" : getRanks("rank:", data.sender).replaceAll("-§r-", "§r§l§7]§r" + "§r§l§7[§r")
    const formated = `§r§l§7[§r${rank}§r§l§7]§r §h${data.sender.name}§r: §g${data.message}`
    world.sendMessage(formated)
})

const getRanks = (string, player) => {
    return player.getTags().filter(tag => tag.startsWith(string)).map(tag => tag.substring(string.length)).join("-§r-") ?? "§fOyuncu"
}

world.beforeEvents.itemUse.subscribe(data => {
    if (data.itemStack.typeId == "minecraft:name_tag" && data.source.hasTag("admin") && data.source.isSneaking) {
        {
            const player = data.source;
            system.run(() => rank_menu(player))
        }

    }
})

/**
 * 
 * @param {Player} player 
 */
async function rank_menu(player) {
    const players = world.getAllPlayers();
    const form = new ActionFormData()
        .title('Rütbe Ekle')
        .body(`
§eRütbe vermek istediğin oyuncuyu seç!
            `)

    players.forEach(p => {
        form.button(`${p.name}`)
    })
    form.button('Çık')
        .show(player).then(res => {
            if (res.canceled || res.selection === undefined) return;
            if (res.selection === players.length) return;
            players.forEach((p, i) => {
                if (res.selection === i) {
                    rank_remove(player, p)
                }
            })
        })
}

/**
 * 
 * @param {Player} player 
 * @param {Player} targetPlayer 
 */
async function rank_remove(player, targetPlayer) {
    const ranks = targetPlayer.getTags().filter(tag => tag.includes('rank:'))
    const form = new ActionFormData()
        .title(`Oyuncu Rütbeleri`)

    ranks.forEach(rank => form.button(`${rank.slice(5)}\n§8rütbesini sil!`))

    form.button("§0Yeni Rütbe Ekle§r")
        .show(player).then(res => {
            if (res.canceled || res.selection === undefined) return;
            const rankIndex = res.selection;
            if (rankIndex === ranks.length) rank_add(player, targetPlayer)
            ranks.forEach((v, i) => {
                if (res.selection === i) {
                    system.run(() => {
                        if (targetPlayer.removeTag(ranks[i]))
                            player.sendMessage(`§7[§bSistem§7]§6 §a${ranks[i].slice(ranks[i].indexOf(':') + 1)}§e rütbesi §g${targetPlayer.name} §ekullanıcıdan alındı!`)
                        targetPlayer.sendMessage(`§7[§bSistem§7]§6 §a${ranks[i].slice(ranks[i].indexOf(':') + 1)}§6 rütbesi silindi!`)
                    })
                }
            })
        })

}

/**
 * 
 * @param {Player} player 
 * @param {Player} targetPlayer 
 */
async function rank_add(player, targetPlayer) {
    const form = new ModalFormData()
        .title(`Oyuncu Rütbeleri`)
        .textField(`
§6Kullanıcıdaki Rütbeler:
${targetPlayer.getTags().filter(tag => tag.includes('rank:')).join('§r\n§r').replaceAll('rank:', '')}
            `, `Rütbeyi gir`)
        .submitButton("§0Yeni Rütbe Ekle§r")
        .show(player).then(res => {
            if (res.canceled) return;
            const rank = res.formValues;
            if (rank.toString().trim() === '') return;
            system.run(() => {
                if (targetPlayer.addTag(`rank:${rank}`))
                    player.sendMessage(`§7[§bSistem§7]§6 §a${rank}§e rütbesi §g${targetPlayer.name} §ekullanıcıya verildi!`)
                targetPlayer.sendMessage(`§7[§bSistem§7]§6 §a${rank}§6 rütbesi eklendi`)
            })

        })

}